#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
################################################################################
CaseLevel         CONFORMANCE
CaseAttribute     AUTO
CaseVerboseLevel  DEFAULT
set reportfile    report.csv

#
# test case Name, category, description, GUID...
#
CaseGuid        18702513-5d0d-4775-90ee-30d0f7553bca
CaseName        Release.Conf1.Case2
CaseCategory    DHCP4
CaseDescription {This case is to test the Conformance - EFI_ACCESS_DENIED      \
	              -- DHCP4 Driver is in the Dhcp4Stopped state.}
	              
################################################################################

Include DHCP4/include/Dhcp4.inc.tcl

proc CleanUpEutEnvironment {} {
  Dhcp4ServiceBinding->DestroyChild "@R_Handle, &@R_Status"
  GetAck

  EndScope _DHCP4_RELEASE_CONF1
  EndLog
}

#
# Begin log ...
#
BeginLog
BeginScope  _DHCP4_RELEASE_CONF1

#
# Parameter Definition
# R_ represents "Remote EFI Side Parameter"
# L_ represents "Local OS Side Parameter"
#
UINTN                            R_Status
UINTN                            R_Handle

#
# Call [DHCP4SBP] -> CreateChild to create child.
#
Dhcp4ServiceBinding->CreateChild {&@R_Handle, &@R_Status}
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Dhcp4SBP.CreateChild - Create Child 1"                        \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"
SetVar     [subst $ENTS_CUR_CHILD]  @R_Handle

#
# Call [DHCP4] -> Stop to verify the DHCP Driver in the the Dhcp4Stopped state.
#
Dhcp4->Stop "&@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Dhcp4.Stop - Stop Driver."                                    \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Check Point: Call Dhcp4->Release in the Dhcp4Stopped state.
#
Dhcp4->Release  "&@R_Status"
GetAck
set assert    [VerifyReturnStatus R_Status $EFI_ACCESS_DENIED]
RecordAssertion $assert $Dhcp4ReleaseConf1AssertionGuid002                     \
                "Dhcp4.Release - Call Release while Driver not start, in the   \
                Dhcp4Stopped state"                                            \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_ACCESS_DENIED"

#
# Call Dhcp4->Stop to stop the driver
#
Dhcp4->Stop "&@R_Status"
GetAck
set assert [VerifyReturnStatus R_Status $EFI_SUCCESS]
RecordAssertion $assert $GenericAssertionGuid                                  \
                "Dhcp4.Stop - Stop Driver."                                    \
                "ReturnStatus - $R_Status, ExpectedStatus - $EFI_SUCCESS"

#
# Clean up the environment on EUT side.
#
CleanUpEutEnvironment