/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Copyright (c) 2006, Intel Corporation
All rights reserved. 

This UEFI SCT ("Software") is furnished under license and may only be used or copied
in accordance with the terms of that license UEFI Contributor Agreement.
No license, express or implied, by estoppel or otherwise, to any intellectual property
rights is granted by this document. The Software is subject to change without notice,
and should not be construed as a commitment by Intel Corporation to market, license,
sell or support any product or technology. Unless otherwise provided for in the license
under which this Software is provided, the Software is provided AS IS, with no warranties
of any kind, express or implied. Except as expressly permitted by the Software license,
neither Intel Corporation nor its suppliers assumes any responsibility or liability
for any errors or inaccuracies that may appear herein. 

Module Name:

  MemoryAllocationENTSTestCase.c

Abstract:

--*/

#include "MemoryAllocationENTSTestCase.h"

//
// gBS.AllocatePages
//
static EFI_ALLOCATE_TYPE    BSAllocatePagesType;
static EFI_MEMORY_TYPE      BSAllocatePagesMemoryType;
static UINTN                BSAllocatePagesPages;
static EFI_PHYSICAL_ADDRESS *BSAllocatePagesMemory;
static EFI_STATUS           *BSAllocatePagesStatus;

ENTS_ARG_FIELD              gBSAllocatePagesArgField[] = {
  {
    OCTETN,
    &BSAllocatePagesType
  },
  {
    OCTETN,
    &BSAllocatePagesMemoryType
  },
  {
    OCTETN,
    &BSAllocatePagesPages
  },
  {
    POINTER,
    &BSAllocatePagesMemory
  },
  {
    POINTER,
    &BSAllocatePagesStatus
  },
  0
};

EFI_STATUS
BS_AllocatePages_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

 AllocatePages Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSAllocatePagesStatus = BootService->AllocatePages (
                                          BSAllocatePagesType,
                                          BSAllocatePagesMemoryType,
                                          BSAllocatePagesPages,
                                          BSAllocatePagesMemory
                                          );
  return EFI_SUCCESS;
}

//
// gBS.FreePages
//
static EFI_PHYSICAL_ADDRESS BSFreePagesMemory;
static UINTN                BSFreePagesPages;
static EFI_STATUS           *BSFreePagesStatus;

ENTS_ARG_FIELD              gBSFreePagesArgField[] = {
  {
    OCTETN,
    &BSFreePagesMemory
  },
  {
    OCTETN,
    &BSFreePagesPages
  },
  {
    POINTER,
    &BSFreePagesStatus
  },
  0
};

EFI_STATUS
BS_FreePages_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

 FreePages Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSFreePagesStatus = BootService->FreePages (
                                      BSFreePagesMemory,
                                      BSFreePagesPages
                                      );
  return EFI_SUCCESS;
}

//
// gBS.AllocatePool
//
static EFI_MEMORY_TYPE  BSAllocatePoolPoolType;
static UINTN            BSAllocatePoolSize;
static VOID             **BSAllocatePoolBuffer;
static EFI_STATUS       *BSAllocatePoolStatus;

ENTS_ARG_FIELD          gBSAllocatePoolArgField[] = {
  {
    OCTETN,
    &BSAllocatePoolPoolType
  },
  {
    OCTETN,
    &BSAllocatePoolSize
  },
  {
    POINTER,
    &BSAllocatePoolBuffer
  },
  {
    POINTER,
    &BSAllocatePoolStatus
  },
  0
};

EFI_STATUS
BS_AllocatePool_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

 AllocatePool Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSAllocatePoolStatus = BootService->AllocatePool (
                                        BSAllocatePoolPoolType,
                                        BSAllocatePoolSize,
                                        BSAllocatePoolBuffer
                                        );
  return EFI_SUCCESS;
}

//
// gBS.FreePool
//
static VOID       *BSFreePoolBuffer;
static EFI_STATUS *BSFreePoolStatus;

ENTS_ARG_FIELD    gBSFreePoolArgField[] = {
  {
    POINTER,
    &BSFreePoolBuffer
  },
  {
    POINTER,
    &BSFreePoolStatus
  },
  0
};

EFI_STATUS
BS_FreePool_EntsTest (
  VOID *ClientInterface
  )
/*++

Routine Description:

 FreePool Ents test entry point.

Arguments:

  ClientInterface - RPC calling interface.

Returns:

  EFI_SUCCESS - Operation succeeded.

--*/
{
  EFI_BOOT_SERVICES *BootService;

  BootService       = (EFI_BOOT_SERVICES *) ClientInterface;

  *BSFreePoolStatus = BootService->FreePool (BSFreePoolBuffer);
  return EFI_SUCCESS;
}
