/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
    HIIFontBBTestFunction.c

Abstract:
    for EFI Driver HII Font Protocol's function Test

--*/
#include "HIIFontBBTestMain.h"

extern UINT8 mPackageList1[];

EFI_STATUS
BBTestStringToImageFunctionTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  );

EFI_STATUS
BBTestStringToImageFunctionTestCheckpoint2 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont,
  IN EFI_GRAPHICS_OUTPUT_PROTOCOL               *GraphicsOutput
  );

EFI_STATUS
BBTestStringIdToImageFunctionTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_DATABASE_PROTOCOL                  *HIIDatabase,
  IN EFI_HII_STRING_PROTOCOL                    *HIIString,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  );

EFI_STATUS
BBTestStringIdToImageFunctionTestCheckpoint2 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_DATABASE_PROTOCOL                  *HIIDatabase,
  IN EFI_HII_STRING_PROTOCOL                    *HIIString,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont,
  IN EFI_GRAPHICS_OUTPUT_PROTOCOL               *GraphicsOutput
  );

EFI_STATUS
BBTestGetGlyphFunctionTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  );

EFI_STATUS
BBTestGetFontInfoFunctionTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  );

EFI_STATUS
BBTestGetFontInfoFunctionTestCheckpoint2 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  );

EFI_STATUS
BBTestStringToImageFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  )
{
  EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib;
  EFI_STATUS                            Status;
  EFI_HII_FONT_PROTOCOL                 *HIIFont;
  EFI_GRAPHICS_OUTPUT_PROTOCOL          *GraphicsOutput;

  //
  // init
  //
  HIIFont = (EFI_HII_FONT_PROTOCOL*)ClientInterface;

  //
  // Get the Standard Library Interface
  //
  Status = gtBS->HandleProtocol (
                   SupportHandle,
                   &gEfiStandardTestLibraryGuid,
                   &StandardLib
                   );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  Status = GetGraphicsOutputInterface ( &GraphicsOutput );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  //
  //Call check points
  //
  BBTestStringToImageFunctionTestCheckpoint1 (StandardLib, HIIFont);

  BBTestStringToImageFunctionTestCheckpoint2 (StandardLib, HIIFont, GraphicsOutput);

  
  return EFI_SUCCESS;
}


EFI_STATUS
BBTestStringIdToImageFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  )
{
  EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib;
  EFI_STATUS                            Status;
  EFI_HII_FONT_PROTOCOL                 *HIIFont;
  EFI_HII_STRING_PROTOCOL               *HIIString;
  EFI_HII_DATABASE_PROTOCOL             *HIIDatabase;
  EFI_GRAPHICS_OUTPUT_PROTOCOL          *GraphicsOutput;

  //
  // init
  //
  HIIFont = (EFI_HII_FONT_PROTOCOL*)ClientInterface;

  //
  // Get the Standard Library Interface
  //
  Status = gtBS->HandleProtocol (
                   SupportHandle,
                   &gEfiStandardTestLibraryGuid,
                   &StandardLib
                   );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }
  
  Status = GetHIIDatabaseInterface( &HIIDatabase );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  Status = GetHIIStringInterface ( &HIIString );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  Status = GetGraphicsOutputInterface ( &GraphicsOutput );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }
  
  //
  //Call check points
  //
  BBTestStringIdToImageFunctionTestCheckpoint1 (StandardLib, HIIDatabase, HIIString, HIIFont);

  BBTestStringIdToImageFunctionTestCheckpoint2 (StandardLib, HIIDatabase, HIIString, HIIFont, GraphicsOutput);
  
  return EFI_SUCCESS;
}


EFI_STATUS
BBTestGetGlyphFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  )
{
  EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib;
  EFI_STATUS                            Status;
  EFI_HII_FONT_PROTOCOL                 *HIIFont;

  //
  // init
  //
  HIIFont = (EFI_HII_FONT_PROTOCOL*)ClientInterface;

  //
  // Get the Standard Library Interface
  //
  Status = gtBS->HandleProtocol (
                   SupportHandle,
                   &gEfiStandardTestLibraryGuid,
                   &StandardLib
                   );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  //
  //Call check points
  //

  BBTestGetGlyphFunctionTestCheckpoint1 (StandardLib, HIIFont);
  
  return EFI_SUCCESS;
}


EFI_STATUS
BBTestGetFontInfoFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  )
{
  EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib;
  EFI_STATUS                            Status;
  EFI_HII_FONT_PROTOCOL                 *HIIFont;

  //
  // init
  //
  HIIFont = (EFI_HII_FONT_PROTOCOL*)ClientInterface;

  //
  // Get the Standard Library Interface
  //
  Status = gtBS->HandleProtocol (
                   SupportHandle,
                   &gEfiStandardTestLibraryGuid,
                   &StandardLib
                   );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  //
  //Call check points
  //
  BBTestGetFontInfoFunctionTestCheckpoint1 (StandardLib, HIIFont);

  BBTestGetFontInfoFunctionTestCheckpoint2 (StandardLib, HIIFont);
  
  return EFI_SUCCESS;
}



EFI_STATUS
BBTestStringToImageFunctionTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  )
{
  EFI_STATUS            Status;
  EFI_TEST_ASSERTION    AssertionType;
  
  EFI_STRING               TestString;
  EFI_FONT_DISPLAY_INFO    *StringInfo;
  EFI_IMAGE_OUTPUT         *Blt;
  UINTN                    BltX;
  UINTN                    BltY;
  EFI_HII_ROW_INFO         **RowInfoArray = NULL;
  UINTN                    *RowInfoArraySize = NULL;
  UINTN                    *ColumnInfoArray = NULL;
  UINTN                    Index = 0;
  
  //
  // Init the Flags, String, StringInfo, Blt
  //
  EFI_HII_OUT_FLAGS ValidFlags[] = {
    EFI_HII_OUT_FLAG_CLIP, 
    EFI_HII_OUT_FLAG_WRAP,
    EFI_HII_OUT_FLAG_CLIP_CLEAN_Y | EFI_HII_OUT_FLAG_CLIP,
    EFI_HII_OUT_FLAG_CLIP_CLEAN_X | EFI_HII_OUT_FLAG_CLIP,
    EFI_HII_OUT_FLAG_TRANSPARENT,
    EFI_HII_IGNORE_IF_NO_GLYPH,
    EFI_HII_IGNORE_LINE_BREAK,
    0
  };
  Blt = (EFI_IMAGE_OUTPUT *) AllocateZeroPool (sizeof(EFI_IMAGE_OUTPUT));
  if ( Blt == NULL ){
    return  EFI_UNSUPPORTED;
  }

  Blt->Width = 50;
  Blt->Height = 40;
  
  Blt->Image.Bitmap = (EFI_GRAPHICS_OUTPUT_BLT_PIXEL *)AllocateZeroPool (Blt->Width * Blt->Height * sizeof(EFI_GRAPHICS_OUTPUT_BLT_PIXEL));
  if ( Blt->Image.Bitmap == NULL ){
  	gtBS->FreePool (Blt);
    return  EFI_UNSUPPORTED;
  }
  
  TestString = L"TestString";
  StringInfo = NULL;
  BltX = 0;
  BltY = 0;

  //
  // Call HIIFont->StringToImage with valid parameters
  //
  for ( Index = 0; ValidFlags[Index] != 0; Index++ ){
    Status = HIIFont->StringToImage (
                        HIIFont,
                        ValidFlags[Index],
                        TestString,
                        StringInfo,
                        &Blt,
                        BltX,
                        BltY,
                        RowInfoArray,
                        RowInfoArraySize,
                        ColumnInfoArray
                        );
                            

    if ( EFI_SUCCESS != Status ) {
      AssertionType = EFI_TEST_ASSERTION_FAILED;
    } else {
      AssertionType = EFI_TEST_ASSERTION_PASSED;
    }
    StandardLib->RecordAssertion (
                   StandardLib,
                   AssertionType,
                   gHIIFontBBTestFunctionAssertionGuid001,
                   L"HII_FONT_PROTOCOL.StringToImage - StringToImage() returns EFI_SUCCESS with valid parameters.",
                   L"%a:%d: Status - %r",
                   __FILE__,
                   (UINTN)__LINE__,
                   Status
                   );

	if ( RowInfoArray != NULL )
		gtBS->FreePool (RowInfoArray);
  }

  gtBS->FreePool (Blt->Image.Bitmap);
  gtBS->FreePool (Blt);
  
  return EFI_SUCCESS;
}


EFI_STATUS
BBTestStringToImageFunctionTestCheckpoint2 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont,
  IN EFI_GRAPHICS_OUTPUT_PROTOCOL               *GraphicsOutput
  )
{
  EFI_STATUS            Status;
  EFI_TEST_ASSERTION    AssertionType;
  
  EFI_STRING               TestString;
  EFI_FONT_DISPLAY_INFO    *StringInfo;
  EFI_IMAGE_OUTPUT         *Blt;
  UINTN                    BltX;
  UINTN                    BltY;
  EFI_HII_ROW_INFO         **RowInfoArray = NULL;
  UINTN                    *RowInfoArraySize = NULL;
  UINTN                    *ColumnInfoArray = NULL;
  UINTN                    Index = 0;

  
  UINT32                   HorizontalResolution;
  UINT32                   VerticalResolution;
  
  //
  // Init the Flags, String, StringInfo, Blt
  //
  EFI_HII_OUT_FLAGS ValidFlags[] = {
    EFI_HII_OUT_FLAG_CLIP | EFI_HII_DIRECT_TO_SCREEN, 
    EFI_HII_OUT_FLAG_WRAP | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_OUT_FLAG_CLIP_CLEAN_Y | EFI_HII_OUT_FLAG_CLIP | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_OUT_FLAG_CLIP_CLEAN_X | EFI_HII_OUT_FLAG_CLIP | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_IGNORE_IF_NO_GLYPH | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_IGNORE_LINE_BREAK | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_DIRECT_TO_SCREEN,
    0
  };

  if ( GraphicsOutput != NULL ) {
    HorizontalResolution = GraphicsOutput->Mode->Info->HorizontalResolution;
    VerticalResolution = GraphicsOutput->Mode->Info->VerticalResolution;
  } else {
    return EFI_UNSUPPORTED;
  }

  Blt = (EFI_IMAGE_OUTPUT *) AllocateZeroPool (sizeof(EFI_IMAGE_OUTPUT));
  if ( Blt == NULL ){
    return  EFI_UNSUPPORTED;
  }
  
  Blt->Width = (UINT16) (HorizontalResolution);
  Blt->Height = (UINT16) (VerticalResolution);
  Blt->Image.Screen = GraphicsOutput;
  TestString = L"TestString";
  StringInfo = NULL;
  BltX = 0;
  BltY = 0;

  //
  // Call HIIFont->StringToImage with valid parameters
  //
  for ( Index = 0; ValidFlags[Index] != 0; Index++ ){
    Status = HIIFont->StringToImage (
                        HIIFont,
                        ValidFlags[Index],
                        TestString,
                        StringInfo,
                        &Blt,
                        BltX,
                        BltY,
                        RowInfoArray,
                        RowInfoArraySize,
                        ColumnInfoArray
                        );
                            

    if ( EFI_SUCCESS != Status ) {
      AssertionType = EFI_TEST_ASSERTION_FAILED;
    } else {
      AssertionType = EFI_TEST_ASSERTION_PASSED;
    }
    StandardLib->RecordAssertion (
                   StandardLib,
                   AssertionType,
                   gHIIFontBBTestFunctionAssertionGuid002,
                   L"HII_FONT_PROTOCOL.StringToImage - StringToImage() returns EFI_SUCCESS with valid parameters.",
                   L"%a:%d: Status - %r",
                   __FILE__,
                   (UINTN)__LINE__,
                   Status
                   );

	if ( RowInfoArray != NULL )
		gtBS->FreePool (RowInfoArray);
  }

  gtBS->FreePool (Blt);
  
  return EFI_SUCCESS;
}



EFI_STATUS
BBTestStringIdToImageFunctionTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_DATABASE_PROTOCOL                  *HIIDatabase,
  IN EFI_HII_STRING_PROTOCOL                    *HIIString,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  )
{
  EFI_STATUS            Status;
  EFI_TEST_ASSERTION    AssertionType;

  EFI_HII_HANDLE                 Handle;
  EFI_HII_PACKAGE_LIST_HEADER    *PackageList;
  UINTN                          Index;
  
  EFI_STRING_ID            StringId;
  CHAR8                    *Language;
  CHAR16                   *LanguageName;
  EFI_FONT_DISPLAY_INFO    *StringInfo;
  EFI_IMAGE_OUTPUT         *Blt;
  UINTN                    BltX;
  UINTN                    BltY;

  EFI_STRING               TestString;
  EFI_HII_ROW_INFO         **RowInfoArray = NULL;
  UINTN                    *RowInfoArraySize = NULL;
  UINTN                    *ColumnInfoArray = NULL;

  EFI_HII_OUT_FLAGS ValidFlags[] = {
    EFI_HII_OUT_FLAG_CLIP, 
    EFI_HII_OUT_FLAG_WRAP,
    EFI_HII_OUT_FLAG_CLIP_CLEAN_Y | EFI_HII_OUT_FLAG_CLIP,
    EFI_HII_OUT_FLAG_CLIP_CLEAN_X | EFI_HII_OUT_FLAG_CLIP,
    EFI_HII_OUT_FLAG_TRANSPARENT,
    EFI_HII_IGNORE_IF_NO_GLYPH,
    EFI_HII_IGNORE_LINE_BREAK,
    0
  };
  
  //
  // Allocate Memory
  //
  Blt = (EFI_IMAGE_OUTPUT *) AllocateZeroPool (sizeof(EFI_IMAGE_OUTPUT));
  if ( Blt == NULL ){
    return  EFI_UNSUPPORTED;
  }

  Blt->Width = 50;
  Blt->Height = 40;
  Blt->Image.Bitmap = (EFI_GRAPHICS_OUTPUT_BLT_PIXEL *)AllocateZeroPool (Blt->Width *Blt->Height * sizeof(EFI_GRAPHICS_OUTPUT_BLT_PIXEL));
  if ( Blt->Image.Bitmap == NULL ){
  	gtBS->FreePool (Blt);
    return  EFI_UNSUPPORTED;
  } 

  //
  // Add the PackageList to the HII database to get a EFI_HII_HANDLE
  //
  PackageList = (EFI_HII_PACKAGE_LIST_HEADER*)mPackageList1;

  Status = HIIDatabase->NewPackageList(
                          HIIDatabase,
                          PackageList,
                          NULL,
                          &Handle
                          );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  //
  // Add a String to the PackageList to get a StringId
  //
  Language = "en-US";
  TestString = L"TestString";
  LanguageName = L"English(United States)";
  Status = HIIString->NewString(
                        HIIString,
                        Handle,
                        &StringId,
                        Language,
                        LanguageName,
                        TestString,
                        NULL);
  if ( EFI_ERROR(Status) ) {
    return Status;
  }
  
  StringInfo = NULL;
  BltX = 0;
  BltY = 0;
  
  //
  // Call HIIFont->StringIdToImage with valid parameters
  //
  for ( Index = 0; ValidFlags[Index] != 0; Index++ ){
    Status = HIIFont->StringIdToImage (
                        HIIFont,
                        ValidFlags[Index],
                        Handle,
                        StringId,
                        Language,
                        StringInfo,
                        &Blt,
                        BltX,
                        BltY,
                        RowInfoArray,
                        RowInfoArraySize,
                        ColumnInfoArray
                        );
                            

    if ( EFI_SUCCESS != Status ) {
      AssertionType = EFI_TEST_ASSERTION_FAILED;
    } else {
      AssertionType = EFI_TEST_ASSERTION_PASSED;
    }
    StandardLib->RecordAssertion (
                   StandardLib,
                   AssertionType,
                   gHIIFontBBTestFunctionAssertionGuid003,
                   L"HII_FONT_PROTOCOL.StringIdToImage - StringIdToImage() returns EFI_SUCCESS with valid parameters.",
                   L"%a:%d: Status - %r",
                   __FILE__,
                   (UINTN)__LINE__,
                   Status
                   );

	if (RowInfoArray != NULL)
		gtBS->FreePool (RowInfoArray);
  }

  gtBS->FreePool (Blt->Image.Bitmap);
  gtBS->FreePool (Blt);

  //
  // Remove the PackageList from the HII database
  //
  Status = HIIDatabase->RemovePackageList (
                          HIIDatabase,
                          Handle
                          );

  return EFI_SUCCESS;
}


EFI_STATUS
BBTestStringIdToImageFunctionTestCheckpoint2 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_DATABASE_PROTOCOL                  *HIIDatabase,
  IN EFI_HII_STRING_PROTOCOL                    *HIIString,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont,
  IN EFI_GRAPHICS_OUTPUT_PROTOCOL               *GraphicsOutput
  )
{
  EFI_STATUS            Status;
  EFI_TEST_ASSERTION    AssertionType;

  EFI_HII_HANDLE                 Handle;
  EFI_HII_PACKAGE_LIST_HEADER    *PackageList;
  UINTN                          Index;
  
  EFI_STRING_ID            StringId;
  CHAR8                    *Language;
  CHAR16                   *LanguageName;
  EFI_FONT_DISPLAY_INFO    *StringInfo;
  EFI_IMAGE_OUTPUT         *Blt;
  UINTN                    BltX;
  UINTN                    BltY;

  EFI_STRING               TestString;
  EFI_HII_ROW_INFO         **RowInfoArray = NULL;
  UINTN                    *RowInfoArraySize = NULL;
  UINTN                    *ColumnInfoArray = NULL;
  
  UINT32                   HorizontalResolution;
  UINT32                   VerticalResolution;
  

  EFI_HII_OUT_FLAGS ValidFlags[] = {
    EFI_HII_OUT_FLAG_CLIP | EFI_HII_DIRECT_TO_SCREEN, 
    EFI_HII_OUT_FLAG_WRAP | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_OUT_FLAG_CLIP_CLEAN_Y | EFI_HII_OUT_FLAG_CLIP | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_OUT_FLAG_CLIP_CLEAN_X | EFI_HII_OUT_FLAG_CLIP | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_IGNORE_IF_NO_GLYPH | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_IGNORE_LINE_BREAK | EFI_HII_DIRECT_TO_SCREEN,
    EFI_HII_DIRECT_TO_SCREEN,
    0
  };

  if ( GraphicsOutput != NULL ) {
    HorizontalResolution = GraphicsOutput->Mode->Info->HorizontalResolution;
    VerticalResolution = GraphicsOutput->Mode->Info->VerticalResolution;
  } else {
    return EFI_UNSUPPORTED;
  }

  //
  // Add the PackageList to the HII database to get a EFI_HII_HANDLE
  //
  PackageList = (EFI_HII_PACKAGE_LIST_HEADER*)mPackageList1;

  Status = HIIDatabase->NewPackageList(
                          HIIDatabase,
                          PackageList,
                          NULL,
                          &Handle
                          );
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  //
  // Add a String to the PackageList to get a StringId
  //
  Language = "en-US";
  TestString = L"TestString";
  LanguageName = L"English(United States)";
  Status = HIIString->NewString(
                        HIIString,
                        Handle,
                        &StringId,
                        Language,
                        LanguageName,
                        TestString,
                        NULL);
  if ( EFI_ERROR(Status) ) {
    return Status;
  }

  //
  // Allocate Memory
  //
  Blt = (EFI_IMAGE_OUTPUT *) AllocateZeroPool (sizeof(EFI_IMAGE_OUTPUT));
  if ( Blt == NULL ){
    return  EFI_UNSUPPORTED;
  }
  
  
  StringInfo = NULL;
  Blt->Width = (UINT16) (HorizontalResolution);
  Blt->Height = (UINT16) (VerticalResolution);
  Blt->Image.Screen = GraphicsOutput;
  BltX = 0;
  BltY = 0;
  
  //
  // Call HIIFont->StringIdToImage with valid parameters
  //
  for ( Index = 0; ValidFlags[Index] != 0; Index++ ){
    Status = HIIFont->StringIdToImage (
                        HIIFont,
                        ValidFlags[Index],
                        Handle,
                        StringId,
                        Language,
                        StringInfo,
                        &Blt,
                        BltX,
                        BltY,
                        RowInfoArray,
                        RowInfoArraySize,
                        ColumnInfoArray
                        );
                            

    if ( EFI_SUCCESS != Status ) {
      AssertionType = EFI_TEST_ASSERTION_FAILED;
    } else {
      AssertionType = EFI_TEST_ASSERTION_PASSED;
    }
    StandardLib->RecordAssertion (
                   StandardLib,
                   AssertionType,
                   gHIIFontBBTestFunctionAssertionGuid004,
                   L"HII_FONT_PROTOCOL.StringIdToImage - StringIdToImage() returns EFI_SUCCESS with valid parameters.",
                   L"%a:%d: Status - %r",
                   __FILE__,
                   (UINTN)__LINE__,
                   Status
                   );

	if ( RowInfoArray != NULL )
		gtBS->FreePool (RowInfoArray);
  }

  gtBS->FreePool (Blt);

  //
  // Remove the PackageList from the HII database
  //
  Status = HIIDatabase->RemovePackageList (
                          HIIDatabase,
                          Handle
                          );

  return EFI_SUCCESS;
}


EFI_STATUS
BBTestGetGlyphFunctionTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  )
{
  EFI_STATUS            Status;
  EFI_TEST_ASSERTION    AssertionType;
  
  CHAR16                   Char = L'A';
  EFI_FONT_DISPLAY_INFO    *StringInfo = NULL;
  EFI_IMAGE_OUTPUT         *Blt = NULL;
  UINTN                    Baseline = 0;
  
  //
  // Call GetGlyph with valid parameters
  //
  Status = HIIFont->GetGlyph (
                      HIIFont,
                      Char,
                      StringInfo,
                      &Blt,
                      &Baseline
                      );

  if ( EFI_SUCCESS == Status ){
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  }else if ( EFI_WARN_UNKNOWN_GLYPH == Status ){
    AssertionType = EFI_TEST_ASSERTION_WARNING;
  }else if ( EFI_OUT_OF_RESOURCES == Status ) {
    AssertionType = EFI_TEST_ASSERTION_WARNING;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;

  }
  StandardLib->RecordAssertion (
                 StandardLib,
                 AssertionType,
                 gHIIFontBBTestFunctionAssertionGuid005,
                 L"HII_FONT_PROTOCOL.GetGlyph - GetGlyph() returns EFI_SUCCESS with valid parameters.",
                 L"%a:%d: Status - %r",
                 __FILE__,
                 (UINTN)__LINE__,
                 Status
                 );
  
  if ( EFI_SUCCESS == Status ){
  	gtBS->FreePool (Blt->Image.Bitmap);
    gtBS->FreePool (Blt);
  }

  return EFI_SUCCESS;
}


EFI_STATUS
BBTestGetFontInfoFunctionTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  )
{

  EFI_STATUS            Status;
  EFI_TEST_ASSERTION    AssertionType;
  
  EFI_FONT_DISPLAY_INFO    *StringInfoIn;
  EFI_FONT_DISPLAY_INFO    *StringInfoOut;
  EFI_STRING               String = L"TestString";
  EFI_FONT_HANDLE          FontHandle = NULL;

  StringInfoOut = NULL;
  //
  // Init 
  StringInfoIn = (EFI_FONT_DISPLAY_INFO *) AllocateZeroPool (sizeof(EFI_FONT_DISPLAY_INFO));
  if ( StringInfoIn == NULL ){
    return  EFI_UNSUPPORTED;
  }
  
  StringInfoIn->FontInfo.FontStyle = EFI_FONT_INFO_SYS_FONT;
  StringInfoIn->FontInfo.FontSize = 20;
  StringInfoIn->ForegroundColor.Blue= 0;
  StringInfoIn->ForegroundColor.Green = 0;
  StringInfoIn->ForegroundColor.Red = 0;
  StringInfoIn->BackgroundColor.Blue = 150;
  StringInfoIn->BackgroundColor.Green = 150;
  StringInfoIn->BackgroundColor.Red = 150;
  StringInfoIn->FontInfoMask = EFI_FONT_INFO_SYS_FONT | EFI_FONT_INFO_SYS_SIZE | EFI_FONT_INFO_SYS_STYLE | \
                               EFI_FONT_INFO_SYS_FORE_COLOR | EFI_FONT_INFO_SYS_BACK_COLOR;

  Status = EFI_SUCCESS;

  while( !EFI_ERROR(Status) ){

    Status = HIIFont->GetFontInfo (
                        HIIFont,
                        &FontHandle,
                        StringInfoIn,
                        &StringInfoOut,
                        String
                        );
	
    if ( StringInfoOut != NULL ) {
      gtBS->FreePool (StringInfoOut);
      StringInfoOut = NULL;
    }
  }	
  
  if ( (EFI_NOT_FOUND == Status && NULL == FontHandle) || EFI_OUT_OF_RESOURCES == Status) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }
  
  StandardLib->RecordAssertion (
                 StandardLib,
                 AssertionType,
                 gHIIFontBBTestFunctionAssertionGuid006,
                 L"HII_FONT_PROTOCOL.GetFont - GetFont() returns EFI_SUCCESS with valid parameters.",
                 L"%a:%d: Status - %r",
                 __FILE__,
                 (UINTN)__LINE__,
                 Status
                 );
  
  gtBS->FreePool (StringInfoIn);

  return EFI_SUCCESS;
}


EFI_STATUS
BBTestGetFontInfoFunctionTestCheckpoint2 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL         *StandardLib,
  IN EFI_HII_FONT_PROTOCOL                      *HIIFont
  )
{

  EFI_STATUS            Status;
  EFI_TEST_ASSERTION    AssertionType;
  
  EFI_FONT_DISPLAY_INFO    *StringInfoIn;
  EFI_FONT_DISPLAY_INFO    *StringInfoOut;
  EFI_STRING               String = L"TestString";
  EFI_FONT_HANDLE          FontHandle = NULL;


  //When StringInfoIn is NULL, the default system font and color should be returned.
  StringInfoIn = NULL;
  StringInfoOut = NULL;

  Status = EFI_SUCCESS;

  while( !EFI_ERROR(Status) ){

    Status = HIIFont->GetFontInfo (
                        HIIFont,
                        &FontHandle,
                        StringInfoIn,
                        &StringInfoOut,
                        String
                        );
	
    if ( StringInfoOut != NULL ) {
      gtBS->FreePool (StringInfoOut);
      StringInfoOut = NULL;	  
    }
  }	

  
  if ( (EFI_NOT_FOUND == Status && NULL == FontHandle) || EFI_OUT_OF_RESOURCES == Status) {
    AssertionType = EFI_TEST_ASSERTION_PASSED;
  } else {
    AssertionType = EFI_TEST_ASSERTION_FAILED;
  }
  
  StandardLib->RecordAssertion (
                 StandardLib,
                 AssertionType,
                 gHIIFontBBTestFunctionAssertionGuid007,
                 L"HII_FONT_PROTOCOL.GetFont - GetFont() returns EFI_SUCCESS with valid parameters.",
                 L"%a:%d: Status - %r",
                 __FILE__,
                 (UINTN)__LINE__,
                 Status
                 );

  return EFI_SUCCESS;
}

