/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
    DriverDiagnosticsBBTestFunction.c

Abstract:
    for EFI Driver Diagnostics Protocol's generic Function Test

--*/

#include "DriverDiagnostics2BBTest.h"

EFI_STATUS
BBTestRunDiagnosticsFuncTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib,
  IN EFI_DRIVER_DIAGNOSTICS_PROTOCOL       *DriverDiagnostics
  );

EFI_STATUS
BBTestRunDiagnosticsFuncTestCheckpoint2 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib,
  IN EFI_DRIVER_DIAGNOSTICS_PROTOCOL       *DriverDiagnostics
  );


/**
 *  @brief Entrypoint for RunDiagnostics() Function Test.
 *         3 checkpoints will be tested.
 *  @param This a pointer of EFI_BB_TEST_PROTOCOL
 *  @param ClientInterface A pointer to the interface array under test
 *  @param TestLevel Test "thoroughness" control
 *  @param SupportHandle A handle containing protocols required
 *  @return EFI_SUCCESS
 *  @return EFI_NOT_FOUND
 */
EFI_STATUS
BBTestRunDiagnosticsFunctionTest (
  IN EFI_BB_TEST_PROTOCOL       *This,
  IN VOID                       *ClientInterface,
  IN EFI_TEST_LEVEL             TestLevel,
  IN EFI_HANDLE                 SupportHandle
  )
{
  EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib;
  EFI_STATUS                            Status;
  EFI_DRIVER_DIAGNOSTICS_PROTOCOL       *DriverDiagnostics;

  //
  // init
  //
  DriverDiagnostics = (EFI_DRIVER_DIAGNOSTICS_PROTOCOL*)ClientInterface;

  //
  // Get the Standard Library Interface
  //
  Status = gtBS->HandleProtocol (
                   SupportHandle,
                   &gEfiStandardTestLibraryGuid,
                   &StandardLib
                   );
  if (EFI_ERROR(Status)) {
    return Status;
  }

  //
  // 7.2.1.2.1  Call RunDiagnostics() with valid ControllerHandle
  //
  BBTestRunDiagnosticsFuncTestCheckpoint1 (StandardLib, DriverDiagnostics);

  //
  // 7.2.1.2.2  Call RunDiagnostics() with valid ChildHandle.
  //
  BBTestRunDiagnosticsFuncTestCheckpoint2 (StandardLib, DriverDiagnostics);

  return EFI_SUCCESS;
}

EFI_STATUS
BBTestRunDiagnosticsFuncTestCheckpoint1 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib,
  IN EFI_DRIVER_DIAGNOSTICS_PROTOCOL       *DriverDiagnostics
  )
{
  EFI_STATUS                                  Status;
  EFI_TEST_ASSERTION                          AssertionType;
  CHAR8                                       *Language = NULL;
  CHAR8                                       Lang[4];
  CHAR8                                       *SupportedLanguages;
  CHAR8                                       *SupportedLanguagesBackup;
  CHAR8                                       *PoolAddress;
  UINTN                                        StringLength;
  UINTN                                       LangIndex;
  UINTN                                       NoHandles;
  EFI_HANDLE                                  *HandleBuffer;
  UINTN                                       SuccessCount;
  CHAR16                                      *ControllerName;
  UINTN                                       Index;
  EFI_HANDLE                                  DriverDiagnosticsHandle = NULL;
  EFI_DRIVER_DIAGNOSTICS_PROTOCOL             *TestedDriverDiagnostics;
  UINTN                                       ControllerHandleCount;
  EFI_HANDLE                                  *ControllerHandleBuffer;
  UINTN                                       Switch = 0;

  EFI_GUID                                    *ErrorType;
  UINTN                                       BufferSize;
  CHAR16                                      *Buffer;
  //
  // locate all component name protocol instances
  //
  Status = gtBS->LocateHandleBuffer (
                   ByProtocol,
                   &gEfiDriverDiagnostics2ProtocolGuid,
                   NULL,
                   &NoHandles,
                   &HandleBuffer
                   );
  if (EFI_ERROR (Status) || (NoHandles == 0)) {
    StandardLib->RecordAssertion (
                   StandardLib,
                   EFI_TEST_ASSERTION_WARNING,
                   gTestGenericFailureGuid,
                   L"Could not locate Driver Diagnostics protocol handles",
                   L"%a:%d: Status - %r, NoHandles - %d",
                   __FILE__,
                   (UINTN)__LINE__,
                   Status,
                   NoHandles
                   );
    return EFI_NOT_FOUND;
  }

  //
  // scan for the handle that matched with the Driver Diagnostics Protocol that
  // passed in by the test framework
  //
  for (Index = 0; Index < NoHandles; Index++) {
    Status = gtBS->HandleProtocol (
                     HandleBuffer[Index],
                     &gEfiDriverDiagnostics2ProtocolGuid,
                     &TestedDriverDiagnostics
                     );
    if (EFI_ERROR (Status)) {
      continue;
    }

    if (TestedDriverDiagnostics == DriverDiagnostics) {
      DriverDiagnosticsHandle = HandleBuffer[Index];
      break;
    }
  }

  gtBS->FreePool (HandleBuffer);

  if (DriverDiagnosticsHandle == NULL) {
    StandardLib->RecordAssertion (
                   StandardLib,
                   EFI_TEST_ASSERTION_WARNING,
                   gTestGenericFailureGuid,
                   L"Could not find the Driver Diagnostics protocol handle - NO",
                   L"%a:%d",
                   __FILE__,
                   (UINTN)__LINE__
                   );
    return EFI_NOT_FOUND;
  }

  //
  // find controller handles managed by the Driver Diagnostics handle.
  //
  Status = LibGetManagedControllerHandles (
             DriverDiagnosticsHandle,
             &ControllerHandleCount,
             &ControllerHandleBuffer
             );
  if (EFI_ERROR (Status)) {
    StandardLib->RecordAssertion (
                   StandardLib,
                   EFI_TEST_ASSERTION_WARNING,
                   gTestGenericFailureGuid,
                   L"Could not get managed controller handle",
                   L"%a:%d",
                   __FILE__,
                   (UINTN)__LINE__
                   );
    return Status;
  }

  LangIndex = 0;
  SupportedLanguages = DriverDiagnostics->SupportedLanguages;
  if (SupportedLanguages == NULL){
    gtBS->FreePool (ControllerHandleBuffer);
    return EFI_UNSUPPORTED;
  }
  
  StringLength = strlena(SupportedLanguages);
  PoolAddress = (CHAR8 *) AllocatePool (StringLength);
  SupportedLanguagesBackup = PoolAddress;
  if (SupportedLanguagesBackup ==NULL){
    gtBS->FreePool (ControllerHandleBuffer);
    return  EFI_UNSUPPORTED;
  }
  EfiAsciiStrCpy(SupportedLanguagesBackup, SupportedLanguages);

  if ((StrChrAscii (SupportedLanguagesBackup, '-') == NULL) &&
  	(StrChrAscii (SupportedLanguagesBackup, ';') == NULL) && 
  	(EfiAsciiStrLen(SupportedLanguagesBackup) >= 3)){
  //ISO639-2, Which belongs to EFI1.1  
    EfiAsciiStrnCpy (Lang, SupportedLanguagesBackup, 3);
    SupportedLanguagesBackup = SupportedLanguagesBackup + 3;
    Switch = 1;	
  }else{
  //RFC3066
    Language = SplitStrAscii(&SupportedLanguagesBackup, ';');
  }
  
 if (Switch == 0){
    while (*Language !='\0') {
  
      SuccessCount = 0;
      for (Index = 0; Index < ControllerHandleCount; Index++) {
  
        ControllerName = NULL;

        Status = DriverDiagnostics->RunDiagnostics (
                                      DriverDiagnostics,
                                      ControllerHandleBuffer[Index],
                                      NULL,
                                      0,
                                      Language,
                                      &ErrorType,
                                      &BufferSize,
                                      &Buffer
                                      );

        if (EFI_SUCCESS == Status) {
          AssertionType = EFI_TEST_ASSERTION_PASSED;
        } else if (EFI_UNSUPPORTED == Status) {
          AssertionType = EFI_TEST_ASSERTION_WARNING;
        } else {
          AssertionType = EFI_TEST_ASSERTION_FAILED;
        }
        StandardLib->RecordAssertion (
                       StandardLib,
                       AssertionType,
                       gFunctionAssertionGuid001,
                       L"EFI_DRIVER_DIAGNOSTICS_PROTOCOL.RunDiagnostics - RunDiagnostics() returns EFI_SUCCESS with supported language",
                       L"%a:%d: Language - %a, ControllerHandleIndex - %d, ControllerName - %s, Status - %r",
                       __FILE__,
                       (UINTN)__LINE__,
                       Language,
                       Index,
                       (ControllerName == NULL) ? L"(NULL)" : ControllerName,
                       Status
                       );
      }
  
      LangIndex++;
      Language = SplitStrAscii (&SupportedLanguagesBackup, ';');
    }
  }else{
    while (Lang[0] !='\0') {

      SuccessCount = 0;
      for (Index = 0; Index < ControllerHandleCount; Index++) {
	  
        ControllerName = NULL;
        Status = DriverDiagnostics->RunDiagnostics (
                                      DriverDiagnostics,
                                      ControllerHandleBuffer[Index],
                                      NULL,
                                      0,
                                      Lang,
                                      &ErrorType,
                                      &BufferSize,
                                      &Buffer
                                      );
        if (EFI_SUCCESS == Status) {
          AssertionType = EFI_TEST_ASSERTION_PASSED;
        } else if (EFI_UNSUPPORTED == Status) {
          AssertionType = EFI_TEST_ASSERTION_WARNING;
        } else {
          AssertionType = EFI_TEST_ASSERTION_FAILED;
        }
        StandardLib->RecordAssertion (
                       StandardLib,
                       AssertionType,
                       gFunctionAssertionGuid001,
                       L"Please Use Native RFC3066 which is preferable in UEFI 2.X\nEFI_DRIVER_DIAGNOSTICS_PROTOCOL.RunDiagnostics - RunDiagnostics() returns EFI_SUCCESS with supported languange",
                       L"%a:%d: LangIndex - %d, Language - %c%c%c, ControllerName - %s, Status - %r",
                       __FILE__,
                       (UINTN)__LINE__,
                       LangIndex,
                       Lang[0],
                       Lang[1],
                       Lang[2],
                       (ControllerName == NULL) ? L"(NULL)" : ControllerName,
                       Status
                       );
	}

      LangIndex++;
      EfiAsciiStrnCpy (Lang, SupportedLanguagesBackup, 3);
      SupportedLanguagesBackup = SupportedLanguagesBackup + 3;
    }
  }

  gtBS->FreePool (ControllerHandleBuffer);
  gtBS->FreePool (PoolAddress);

  return EFI_SUCCESS;
}

EFI_STATUS
BBTestRunDiagnosticsFuncTestCheckpoint2 (
  IN EFI_STANDARD_TEST_LIBRARY_PROTOCOL    *StandardLib,
  IN EFI_DRIVER_DIAGNOSTICS_PROTOCOL       *DriverDiagnostics
  )
{
  EFI_STATUS                                  Status;
  EFI_TEST_ASSERTION                          AssertionType;
  CHAR8                                       *Language = NULL;
  CHAR8                                       Lang[4];
  CHAR8                                       *SupportedLanguages;
  CHAR8                                       *SupportedLanguagesBackup;
  CHAR8                                       *PoolAddress;
  UINTN                                        StringLength;
  UINTN                                       LangIndex;
  UINTN                                       NoHandles;
  EFI_HANDLE                                  *HandleBuffer;
  UINTN                                       Index;
  CHAR16                                      *ControllerName;
  EFI_HANDLE                                  DriverDiagnosticsHandle = NULL;
  EFI_DRIVER_DIAGNOSTICS_PROTOCOL             *TestedDriverDiagnostics;
  UINTN                                       ControllerHandleCount;
  EFI_HANDLE                                  *ControllerHandleBuffer;
  UINTN                                       NumberOfChildren;
  EFI_HANDLE                                  *ChildControllerHandleBuffer;
  UINTN                                       ChildIndex;
  UINTN                                       Switch = 0;
  
  EFI_GUID                                    *ErrorType;
  UINTN                                       BufferSize;
  CHAR16                                      *Buffer;
  //HandleBuffer = NULL;
  //
  // locate all Driver Diagnostics Protocol instances
  //
  Status = gtBS->LocateHandleBuffer (
                   ByProtocol,
                   &gEfiDriverDiagnostics2ProtocolGuid,
                   NULL,
                   &NoHandles,
                   &HandleBuffer
                   );
  if (EFI_ERROR (Status) || (NoHandles == 0)) {
    StandardLib->RecordAssertion (
                   StandardLib,
                   EFI_TEST_ASSERTION_WARNING,
                   gTestGenericFailureGuid,
                   L"Could not locate Driver Diagnostics protocol handles",
                   L"%a:%d: Status - %r, NoHandles - %d",
                   __FILE__,
                   (UINTN)__LINE__,
                   Status,
                   NoHandles
                   );
    return EFI_NOT_FOUND;
  }

  //
  // scan for the handle that matched with the Driver Diagnostics Protocol that
  // passed in by the test framework
  //
  for (Index = 0; Index < NoHandles; Index++) {
    Status = gtBS->HandleProtocol (
                     HandleBuffer[Index],
                     &gEfiDriverDiagnostics2ProtocolGuid,
                     &TestedDriverDiagnostics
                     );
    if (EFI_ERROR (Status)) {
      continue;
    }

    if (TestedDriverDiagnostics == DriverDiagnostics) {
      DriverDiagnosticsHandle = HandleBuffer[Index];
      break;
    }
  }

  gtBS->FreePool (HandleBuffer);

  if (DriverDiagnosticsHandle == NULL) {
    StandardLib->RecordAssertion (
                   StandardLib,
                   EFI_TEST_ASSERTION_WARNING,
                   gTestGenericFailureGuid,
                   L"Could not find the component name2 protocol handle - NO",
                   L"%a:%d",
                   __FILE__,
                   (UINTN)__LINE__
                   );
    return EFI_NOT_FOUND;
  }

  //
  // find controller handles managed by the Driver Diagnostics handle.
  //
  Status = LibGetManagedControllerHandles (
             DriverDiagnosticsHandle,
             &ControllerHandleCount,
             &ControllerHandleBuffer
             );
  if (EFI_ERROR (Status)) {
    StandardLib->RecordAssertion (
                   StandardLib,
                   EFI_TEST_ASSERTION_WARNING,
                   gTestGenericFailureGuid,
                   L"Could not get managed controller handle",
                   L"%a:%d",
                   __FILE__,
                   (UINTN)__LINE__
                   );
    return Status;
  }

  LangIndex = 0;
  SupportedLanguages = DriverDiagnostics->SupportedLanguages;
  if (SupportedLanguages == NULL){
    gtBS->FreePool (ControllerHandleBuffer);
    return  EFI_UNSUPPORTED;
  }	

  StringLength = strlena(SupportedLanguages);
  PoolAddress = (CHAR8 *) AllocatePool (StringLength);
  SupportedLanguagesBackup = PoolAddress;
  if (SupportedLanguagesBackup == NULL){
    gtBS->FreePool (ControllerHandleBuffer);
    return  EFI_UNSUPPORTED;
  }	
  EfiAsciiStrCpy(SupportedLanguagesBackup, SupportedLanguages);

  if ((StrChrAscii (SupportedLanguagesBackup, '-') == NULL) &&
  	(StrChrAscii (SupportedLanguagesBackup, ';') == NULL) && 
  	(EfiAsciiStrLen(SupportedLanguagesBackup) >= 3)){
  //ISO639-2, Which belongs to EFI1.1  
    EfiAsciiStrnCpy (Lang, SupportedLanguagesBackup, 3);
    SupportedLanguagesBackup = SupportedLanguagesBackup + 3;
    Switch = 1;	
  }else{
  //RFC3066
    Language = SplitStrAscii(&SupportedLanguagesBackup, ';');
  }
  
  if (Switch == 0){
  
    while (*Language != '\0') {
  
      for (Index = 0; Index < ControllerHandleCount; Index++) {
  
        ChildControllerHandleBuffer = NULL;
  
        Status = LibGetManagedChildControllerHandles (
                   DriverDiagnosticsHandle,
                   ControllerHandleBuffer[Index],
                   &NumberOfChildren,
                   &ChildControllerHandleBuffer
                   );
        if (EFI_ERROR (Status)) {
          StandardLib->RecordAssertion (
                         StandardLib,
                         EFI_TEST_ASSERTION_WARNING,
                         gTestGenericFailureGuid,
                         L"Could not get managed child controller handle",
                         L"%a:%d",
                         __FILE__,
                         (UINTN)__LINE__
                         );
          continue;
        }
  
        for (ChildIndex = 0; ChildIndex < NumberOfChildren; ChildIndex++) {
  
          ControllerName = NULL;
          Status = DriverDiagnostics->RunDiagnostics (
                                        DriverDiagnostics,
                                        ControllerHandleBuffer[Index],
                                        ChildControllerHandleBuffer[ChildIndex],
                                        0,
                                        Language,
                                        &ErrorType,
                                        &BufferSize,
                                        &Buffer
                                        );		  
          if (EFI_SUCCESS == Status) {
            AssertionType = EFI_TEST_ASSERTION_PASSED;
          } else if ( (EFI_UNSUPPORTED == Status) || (EFI_DEVICE_ERROR == Status) || (EFI_ACCESS_DENIED == Status) ) {
            AssertionType = EFI_TEST_ASSERTION_WARNING;
          } else {
            AssertionType = EFI_TEST_ASSERTION_FAILED;
          }
          StandardLib->RecordAssertion (
                         StandardLib,
                         AssertionType,
                         gFunctionAssertionGuid002,
                         L"EFI_DRIVER_DIAGNOSTICS_PROTOCOL.RunDiagnostics - RunDiagnostics() returns EFI_SUCCESS with supported language",
                         L"%a:%d: Language - %a, ControllerIndex - %d, ChildIndex - %d, ControllerName - %s, Status - %r",
                         __FILE__,
                         (UINTN)__LINE__,
                         Language,
                         Index,
                         ChildIndex,
                         (ControllerName == NULL) ? L"(NULL)" : ControllerName,
                         Status
                         );
        }
  
        if (ChildControllerHandleBuffer != NULL) {
          gtBS->FreePool (ChildControllerHandleBuffer);
        }
      }
  
      LangIndex++;
      Language = SplitStrAscii (&SupportedLanguagesBackup, ';');
    }
  }else {
    while (Lang[0] !='\0') {

      for (Index = 0; Index < ControllerHandleCount; Index++) {
  
        ChildControllerHandleBuffer = NULL;
  
        Status = LibGetManagedChildControllerHandles (
                   DriverDiagnosticsHandle,
                   ControllerHandleBuffer[Index],
                   &NumberOfChildren,
                   &ChildControllerHandleBuffer
                   );
        if (EFI_ERROR (Status)) {
          StandardLib->RecordAssertion (
                         StandardLib,
                         EFI_TEST_ASSERTION_WARNING,
                         gTestGenericFailureGuid,
                         L"Could not get managed child controller handle",
                         L"%a:%d",
                         __FILE__,
                         (UINTN)__LINE__
                         );
          continue;
        }

        for (ChildIndex = 0; ChildIndex < NumberOfChildren; ChildIndex++) {
  
          ControllerName = NULL;
          Status = DriverDiagnostics->RunDiagnostics (
                                        DriverDiagnostics,
                                        ControllerHandleBuffer[Index],
                                        ChildControllerHandleBuffer[ChildIndex],
                                        0,
                                        Lang,
                                        &ErrorType,
                                        &BufferSize,
                                        &Buffer
                                        );	
          if (EFI_SUCCESS == Status) {
            AssertionType = EFI_TEST_ASSERTION_PASSED;
          } else if ( (EFI_UNSUPPORTED == Status) || (EFI_DEVICE_ERROR == Status) || (EFI_ACCESS_DENIED == Status) ) {
            AssertionType = EFI_TEST_ASSERTION_WARNING;
          } else {
            AssertionType = EFI_TEST_ASSERTION_FAILED;
          }
          StandardLib->RecordAssertion (
                         StandardLib,
                         AssertionType,
                         gFunctionAssertionGuid002,
                         L"Please Use Native RFC3066 which is preferable in UEFI 2.X\nEFI_DRIVER_DIAGNOSTICS_PROTOCOL.RunDiagnostics - RunDiagnostics() returns EFI_SUCCESS with supported language",
                         L"%a:%d: Language - %c%c%c, ControllerIndex - %d, ChildIndex - %d, ControllerName - %s, Status - %r",
		                 __FILE__,
                         (UINTN)__LINE__,
                         Lang,
                         Index,
                         ChildIndex,
                         (ControllerName == NULL) ? L"(NULL)" : ControllerName,
                         Status
                         );
        }
  
        if (ChildControllerHandleBuffer != NULL) {
          gtBS->FreePool (ChildControllerHandleBuffer);
        }	
      	}
      LangIndex++;
      EfiAsciiStrnCpy (Lang, SupportedLanguagesBackup, 3);
      SupportedLanguagesBackup = SupportedLanguagesBackup + 3;
    }
  }

  gtBS->FreePool (ControllerHandleBuffer);
  gtBS->FreePool (PoolAddress);

  return EFI_SUCCESS;
}

