/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  ExecuteSupport.h

Abstract:

  This file provides the support services for test execution.

--*/

#ifndef _EFI_EXECUTE_SUPPORT_H_
#define _EFI_EXECUTE_SUPPORT_H_

//
// EFI_SCT_EXECUTE_INFO
//

typedef struct {
  EFI_SCT_TEST_CASE         *TestCase;

  EFI_SCT_CATEGORY_DATA     *Category;
  EFI_SCT_TEST_FILE         *TestFile;

  EFI_SCT_SKIPPED_CASE      *SkippedCase;

  VOID                      *TestProtocol;
  VOID                      *TestEntry;

  EFI_SCT_LOG_STATE         State;

  UINT32                    Index;
  UINT32                    Iteration;
  UINT32                    Passes;
  UINT32                    Warnings;
  UINT32                    Failures;
} EFI_SCT_EXECUTE_INFO;

//
// Support functions declaration
//

EFI_STATUS
InitializeExecuteInfo (
  IN EFI_SCT_TEST_CASE            *TestCase,
  OUT EFI_SCT_EXECUTE_INFO        *ExecuteInfo
  );

EFI_STATUS
GetFileMetaName (
  IN EFI_SCT_EXECUTE_INFO         *ExecuteInfo,
  OUT CHAR16                      **FilePath,
  OUT CHAR16                      **MetaName
  );

EFI_STATUS
GetFileFullMetaName (
  IN EFI_SCT_EXECUTE_INFO         *ExecuteInfo,
  OUT CHAR16                      **FullMetaName
  );

EFI_STATUS
GetKeyFileFullMetaName (
  IN EFI_SCT_EXECUTE_INFO         *ExecuteInfo,
  OUT CHAR16                      **FullMetaName
  );

EFI_STATUS
ResetExecuteTestResults (
  IN EFI_SCT_EXECUTE_INFO         *ExecuteInfo
  );

typedef
BOOLEAN
(EFIAPI *EFI_INTERFACE_FILTER) (
  IN  VOID                              *ClientInterface,
  IN  EFI_HANDLE                        SupportHandle,
  IN  EFI_GUID                          *ProtocolGuid
  );

BOOLEAN
IhvInterfaceFilter (
  IN VOID                       *ClientInterface,
  IN EFI_HANDLE                 SupportHandle,
  IN EFI_GUID                   *ProtocolGuid
  );


#endif
